'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var isEmpty = _interopDefault(require('is-empty'));
var isPlainObject = _interopDefault(require('is-plain-object'));
var slate = require('@gitbook/slate');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
}

const ANCHOR = {};
const CURSOR = {};
const FOCUS = {};
const CREATORS = {
    anchor(tagName, attributes, children) {
        return ANCHOR;
    },
    block(tagName, attributes, children) {
        return slate.Block.create(Object.assign({}, attributes, { nodes: createChildren(children) }));
    },
    cursor(tagName, attributes, children) {
        return CURSOR;
    },
    document(tagName, attributes, children) {
        return slate.Document.create(Object.assign({}, attributes, { nodes: createChildren(children) }));
    },
    focus(tagName, attributes, children) {
        return FOCUS;
    },
    inline(tagName, attributes, children) {
        return slate.Inline.create(Object.assign({}, attributes, { nodes: createChildren(children) }));
    },
    mark(tagName, attributes, children) {
        const marks = slate.Mark.createSet([attributes]);
        const nodes = createChildren(children, { marks });
        return nodes;
    },
    selection(tagName, attributes, children) {
        return slate.Range.create(attributes);
    },
    value(tagName, attributes, children) {
        const { data, normalize = true } = attributes;
        const document = children.find(slate.Document.isDocument);
        let selection = children.find(slate.Range.isRange) || slate.Range.create();
        const props = {};
        if (document) {
            document.getTexts().forEach(text => {
                if (text.__anchor != null) {
                    props.anchorKey = text.key;
                    props.anchorOffset = text.__anchor;
                    props.isFocused = true;
                }
                if (text.__focus != null) {
                    props.focusKey = text.key;
                    props.focusOffset = text.__focus;
                    props.isFocused = true;
                }
            });
        }
        if (props.anchorKey && !props.focusKey) {
            throw new Error(`Slate hyperscript must have both \`<anchor/>\` and \`<focus/>\` defined if one is defined, but you only defined \`<anchor/>\`. For collapsed selections, use \`<cursor/>\`.`);
        }
        if (!props.anchorKey && props.focusKey) {
            throw new Error(`Slate hyperscript must have both \`<anchor/>\` and \`<focus/>\` defined if one is defined, but you only defined \`<focus/>\`. For collapsed selections, use \`<cursor/>\`.`);
        }
        if (!isEmpty(props)) {
            selection = selection.merge(props).normalize(document);
        }
        return slate.Value.fromJS({ data, document, selection }, { normalize });
    },
    text(tagName, attributes, children) {
        const nodes = createChildren(children, { key: attributes.key });
        return nodes;
    }
};
function createHyperscript(options = {}) {
    const creators = resolveCreators(options);
    function create(tagName, attributes, ...children) {
        const creator = creators[tagName];
        if (!creator) {
            throw new Error(`No hyperscript creator found for tag: "${tagName}"`);
        }
        if (attributes == null) {
            attributes = {};
        }
        if (!isPlainObject(attributes)) {
            children = [attributes].concat(children);
            attributes = {};
        }
        children = children
            .filter(child => Boolean(child))
            .reduce((memo, child) => memo.concat(child), []);
        const element = creator(tagName, attributes, children);
        return element;
    }
    return create;
}
function createChildren(children, options = {}) {
    const array = [];
    let length = 0;
    const firstNodeOrText = children.find(c => typeof c !== 'string');
    const firstText = slate.Text.isText(firstNodeOrText) ? firstNodeOrText : null;
    const key = options.key
        ? options.key
        : firstText
            ? firstText.key
            : undefined;
    let node = slate.Text.create({
        key,
        leaves: [{ text: '', marks: options.marks }]
    });
    function setNode(next) {
        const { __anchor, __focus } = node;
        if (__anchor != null) {
            next.__anchor = __anchor;
        }
        if (__focus != null) {
            next.__focus = __focus;
        }
        node = next;
    }
    children.forEach((child, index) => {
        const isLast = index === children.length - 1;
        if (slate.Node.isNode(child) && !slate.Text.isText(child)) {
            if (node.text.length ||
                node.__anchor != null ||
                node.__focus != null ||
                node.getMarksAtIndex(0).size) {
                array.push(node);
            }
            array.push(child);
            node = isLast
                ? null
                : slate.Text.create({ leaves: [{ text: '', marks: options.marks }] });
            length = 0;
        }
        if (typeof child === 'string') {
            setNode(node.insertText(node.text.length, child, options.marks));
            length += child.length;
        }
        if (slate.Text.isText(child)) {
            const { __anchor, __focus } = child;
            let i = node.text.length;
            if (!options.key && node.text.length === 0) {
                setNode(node.set('key', child.key));
            }
            child.getLeaves().forEach(leaf => {
                let { marks } = leaf;
                if (options.marks) {
                    marks = marks.union(options.marks);
                }
                setNode(node.insertText(i, leaf.text, marks));
                i += leaf.text.length;
            });
            if (__anchor != null) {
                node.__anchor = __anchor + length;
            }
            if (__focus != null) {
                node.__focus = __focus + length;
            }
            length += child.text.length;
        }
        if (child === ANCHOR || child === CURSOR) {
            node.__anchor = length;
        }
        if (child === FOCUS || child === CURSOR) {
            node.__focus = length;
        }
    });
    if (node != null) {
        array.push(node);
    }
    return array;
}
function resolveCreators(options) {
    const { blocks = {}, inlines = {}, marks = {} } = options;
    const creators = Object.assign({}, CREATORS, (options.creators || {}));
    Object.keys(blocks).map(key => {
        creators[key] = normalizeNode(key, blocks[key], 'block');
    });
    Object.keys(inlines).map(key => {
        creators[key] = normalizeNode(key, inlines[key], 'inline');
    });
    Object.keys(marks).map(key => {
        creators[key] = normalizeMark(key, marks[key]);
    });
    return creators;
}
function normalizeNode(key, value, object) {
    if (typeof value === 'function') {
        return value;
    }
    if (typeof value === 'string') {
        value = { type: value };
    }
    if (isPlainObject(value)) {
        return (tagName, attributes, children) => {
            const { key: attrKey } = attributes, rest = __rest(attributes, ["key"]);
            const attrs = Object.assign({}, value, { object, key: attrKey, data: Object.assign({}, (value.data || {}), rest) });
            return CREATORS[object](tagName, attrs, children);
        };
    }
    throw new Error(`Slate hyperscript ${object} creators can be either functions, objects or strings, but you passed: ${value}`);
}
function normalizeMark(key, value) {
    if (typeof value === 'function') {
        return value;
    }
    if (typeof value === 'string') {
        value = { type: value };
    }
    if (isPlainObject(value)) {
        return (tagName, attributes, children) => {
            const attrs = Object.assign({}, value, { data: Object.assign({}, (value.data || {}), attributes) });
            return CREATORS.mark(tagName, attrs, children);
        };
    }
    throw new Error(`Slate hyperscript mark creators can be either functions, objects or strings, but you passed: ${value}`);
}
var index = createHyperscript();

exports.default = index;
exports.createHyperscript = createHyperscript;
//# sourceMappingURL=index.js.map
